
#System imports
import sys
import time
import math

#Pymol imports
from pymol import cmd
import pymol

#Wiimol imports
#likely need some shared state here? Who is in control of master is one that works. As well as callbacks neededing access to shared thread information.
import Shared
import Utilities
import SelectionMark





#A few "constants" that can be editted at the top of the file, to tweak values:

Z_SPEED = 10
XY_SPEED = 10


def CheckMaster(controller):
	if(not Shared.master_exists):		
		ToggleMaster(controller)
		Shared.master_exists = True
		return True
	else:
		return controller.master
	

#GUI Manipulation functions:
def SendCursor(controller):
	controller.threed_cursor_coords = Shared.depthTracker.getLocalPOI(controller.pointer_position[0] * Shared.viewport[2] + Shared.viewport[0]
																		, controller.pointer_position[1] * Shared.viewport[3] + Shared.viewport[1])
	if(controller.threed_cursor_coords[2] != 1):
		controller.threed_cursor_visible = True
		controller.threed_cursor_getUnProject = True
		return True
	return False
	
def recompileExtents():
	AllObjects = cmd.get_names("public_objects")
	for i in AllObjects:
		if( not ("/" + i) in Shared.Ignore_Extents):
			extent1 = cmd.get_extent("/" + i)
			try:
				if (not (("/" + i) in Shared.Extent_Dictionary.keys()) or Shared.Extent_Dictionary["/" + i] != extent1 )and (extent1[1][2] - extent1[0][2] > .5):
					molecule = cmd.get_model(i)
					Shared.Extent_Dictionary["/" + i] = cmd.get_extent("/" + i)
					gotChain = []
					gotResi = []
					for atom in molecule.atom:
						if( not atom.chain in gotChain):
							Shared.Extent_Dictionary["/" + i + "//" + atom.chain] = cmd.get_extent("/" + i + "//" + atom.chain)
							gotChain.append(atom.chain)
						if( not atom.chain + "/" + atom.resi in gotResi):
							Shared.Extent_Dictionary["/" + i + "//" + atom.chain + "/" + atom.resi] = cmd.get_extent("/" + i + "//" + atom.chain + "/" + atom.resi)
							gotResi.append(atom.chain + "/" + atom.resi)
						Shared.Extent_Dictionary["/" + i + "//" + atom.chain + "/" + atom.resi + "/" + atom.name] = \
									[[atom.coord[0],atom.coord[1],atom.coord[2]],[atom.coord[0],atom.coord[1],atom.coord[2]]]
			except pymol.CmdException:
				Shared.Ignore_Extents.append("/" + i)

def refreshSelection(kill):
	if(Shared.CurrentSelection == 0):
		for a in range(Shared.SelectionCount):
			cmd.disable("WM_Sel" + str(a + 1))
		if(kill):
			cmd.select("WM_Sel", "none")
		cmd.enable("WM_Sel")
	else:
		for a in range(Shared.SelectionCount):
			if( a + 1 == Shared.CurrentSelection ):
				cmd.enable("WM_Sel" + str(a + 1))
				cmd.select("WM_Sel", "WM_Sel" + str(a + 1))
			else:
				cmd.disable("WM_Sel" + str(a + 1))
def MakeOrderedDepthLine(controller, ray1, ray2):
	if not CheckMaster(controller):
		return
	AllObjects = cmd.get_names("public_objects")
	recompileExtents()
	residueList = []
	for i in AllObjects:
		if( not ("/" + i) in Shared.Ignore_Extents ):
			molecule = cmd.get_model(i)
			for atom in molecule.atom:
				if( not ("/" + i + "//" + atom.chain + "/" + atom.resi) in residueList):
					residueList.append("/" + i + "//" + atom.chain + "/" + atom.resi)
	reducedList = []
	for i in residueList:
		extents = Shared.Extent_Dictionary[str(i)]
		center = [(extents[0][0] + extents[1][0])/2,
					 (extents[0][1] + extents[1][1])/2,
					 (extents[0][2] + extents[1][2])/2]
		project = ProjectToLine(center, ray1, ray2)
		#print [center,ray1,ray2]
		#print project
		shouldAdd = True
		for j in reducedList:
			extentsOfOther = Shared.Extent_Dictionary[j[0]]
			otherCenter = [(extentsOfOther[0][0] + extentsOfOther[1][0])/2,
						 (extentsOfOther[0][1] + extentsOfOther[1][1])/2,
						 (extentsOfOther[0][2] + extentsOfOther[1][2])/2]
			otherProject = ProjectToLine(otherCenter, ray1, ray2)
			if( distance(otherCenter,[project[1],project[2],project[3]]) < project[0]):
				shouldAdd = False
			elif( distance(center,[otherProject[1],otherProject[2],otherProject[3]]) < otherProject[0] ):
				if(shouldAdd):
					j[0] = i
					j[1] = center
				else:	
					reducedList.remove(j)			
				shouldAdd = False
		if(shouldAdd):
			reducedList.append([i, center, project[0]])
	
	#probably should do a quicksort or likewise here, but this is something we will do if speeds are bad and
	# it becomes necessary
	# also, the matrix operation can be optemized.
	finalList = []
	view = cmd.get_view()
	for i in reducedList:
		j = 0
		NotFound = True
		z_depth = MatMult([view[0],view[1],view[2],
					view[3],view[4],view[5],
					view[6],view[7],view[8]],i[1])[2] 
		while NotFound and j < len(finalList):
			if(z_depth < finalList[j][1]):
				finalList.insert(j,[i,z_depth])
				NotFound = False
			j+=1
		if(NotFound):
			finalList.append([i,z_depth])
	finalSpans = []
	#so if  we confine this to two dimensions with the line being y = 0
	# then the distance from the projected point is going to be i[0][2], aka the y-value, and its x-value will be i[1]
	# so we simply need to solve (x1 - t)^2 + y1^2 = (x2 -t)^2 + y2^2 =>
	#									x1^2 - x2^2 - 2*x1*t + 2*x2*t + y1^2 - y2^2 = 0 =>
	#									t = x2^2 - x1^2 + y2^2 - y1^2 / (2( x2 - x1))
	for i in range(len(finalList) - 1):
		finalSpans.append((finalList[i+1][1] + finalList[i][1]) / 2)
	controller.finalList = finalList
	controller.finalSpans = finalSpans
	controller.threed_cursor_view_mat = view
				
def MatMult(mat, vect):
	return [mat[0] * vect[0] + mat[1] * vect[1] + mat[2] * vect[2],
			mat[3] * vect[0] + mat[4] * vect[1] + mat[5] * vect[2],
			mat[6] * vect[0] + mat[7] * vect[1] + mat[8] * vect[2]]
def distance(point1,point2):
	return (math.sqrt(math.pow(point1[0] - point2[0],2)	+ math.pow(point1[1] - point2[1],2) + math.pow(point1[2] - point2[2],2)))
	
def ProjectToLine(point, line1, line2):
	x2 = point[0] - line1[0]
	y2 = point[1] - line1[1]
	z2 = point[2] - line1[2]
	
	x1 = line2[0] - line1[0]
	y1 = line2[1] - line1[1]
	z1 = line2[2] - line1[2]
	
	nx1 = x1/ math.sqrt((x1 * x1) + (y1 * y1) + (z1 * z1))
	ny1 = y1/ math.sqrt((x1 * x1) + (y1 * y1) + (z1 * z1))
	nz1 = z1/ math.sqrt((x1 * x1) + (y1 * y1) + (z1 * z1))
	newpoint = [ x2 * nx1 * nx1 + y2 * nx1 * ny1 + z2 * nx1 * nz1 + line1[0], 
				 x2 * nx1 * ny1 + y2 * ny1 * ny1 + z2 * ny1 * nz1 + line1[1],
				 x2 * nx1 * nz1 + y2 * nz1 * ny1 + z2 * nz1 * nz1 + line1[2]]
	
	return [math.sqrt(math.pow(newpoint[0] - point[0],2) + math.pow(newpoint[1] - point[1],2) + math.pow(newpoint[2] - point[2],2)),
			newpoint[0],newpoint[1],newpoint[2]]
				

def getResiToSnap(controller):
	AllObjects = cmd.get_names("all")	
	selections = cmd.get_names("selections")
	recompileExtents()
	bestObject = None 
	bestObjectDistance = 50000
	#determine best object
	for i in AllObjects: 
		if( not ("/" + i) in Shared.Ignore_Extents and not i in selections ):
			extents = Shared.Extent_Dictionary["/" + str(i)]
			center = [(extents[0][0] + extents[1][0])/2,
						 (extents[0][1] + extents[1][1])/2,
						 (extents[0][2] + extents[1][2])/2]
			if(math.pow(center[0] - controller.threed_cursor_coords[0],2) + \
				math.pow(center[1] - controller.threed_cursor_coords[1],2) + \
				math.pow(center[2] - controller.threed_cursor_coords[2],2) < bestObjectDistance):
				bestObjectDistance = math.pow(center[0] - controller.threed_cursor_coords[0],2) + \
									  math.pow(center[1] - controller.threed_cursor_coords[1],2) + \
									  math.pow(center[2] - controller.threed_cursor_coords[2],2)
				bestObject = i
	bestChain = None
	bestChainDistance = 50000		
	if(bestObject != None):
		for j in Shared.Resi_Dictionary[str(bestObject)].keys():
			extents = Shared.Extent_Dictionary["/" + str(bestObject) + "//" + str(j)]

			center = [(extents[0][0] + extents[1][0])/2,
						 (extents[0][1] + extents[1][1])/2,
						 (extents[0][2] + extents[1][2])/2]
			if(math.pow(center[0] - controller.threed_cursor_coords[0],2) + \
				math.pow(center[1] - controller.threed_cursor_coords[1],2) + \
				math.pow(center[2] - controller.threed_cursor_coords[2],2) < bestChainDistance):
				bestChainDistance = math.pow(center[0] - controller.threed_cursor_coords[0],2) + \
					  math.pow(center[1] - controller.threed_cursor_coords[1],2) + \
					  math.pow(center[2] - controller.threed_cursor_coords[2],2)
				bestChain = j
	bestResi = None
	bestResiDistance = 50000
	if(bestChain != None):
		for k in Shared.Resi_Dictionary[str(bestObject)][str(bestChain)]:
			extents = Shared.Extent_Dictionary["/" + str(bestObject) + "//" + str(bestChain) + "/" + str(k)]
			center = [(extents[0][0] + extents[1][0])/2,
						 (extents[0][1] + extents[1][1])/2,
						 (extents[0][2] + extents[1][2])/2]
						 
			if(math.pow(center[0] - controller.threed_cursor_coords[0],2) + \
				math.pow(center[1] - controller.threed_cursor_coords[1],2) + \
				math.pow(center[2] - controller.threed_cursor_coords[2],2) < bestResiDistance):
				bestResiDistance = math.pow(center[0] - controller.threed_cursor_coords[0],2) + \
						  math.pow(center[1] - controller.threed_cursor_coords[1],2) + \
							math.pow(center[2] - controller.threed_cursor_coords[2],2)
				bestResi = k					
	best_object = ("/" + str(bestObject) + "//" + str(bestChain) + "/" + str(bestResi))
	return best_object

def SnapToResiCursor(controller):	
	if not CheckMaster(controller):
		return
	

	#Selection requires a few bits. Primarily, the
	#Shared.ray[controller.deviceindex]
	if(controller.threed_cursor_moved == False):
		AllObjects = cmd.get_names("all")	
		selections = cmd.get_names("selections")
		recompileExtents()
		bestObject = None 
		bestObjectDistance = 50000
		#determine best object
		for i in AllObjects: 
			if( not ("/" + i) in Shared.Ignore_Extents and not i in selections ):
				extents = Shared.Extent_Dictionary["/" + str(i)]
				center = [(extents[0][0] + extents[1][0])/2,
							 (extents[0][1] + extents[1][1])/2,
							 (extents[0][2] + extents[1][2])/2]
				if(math.pow(center[0] - controller.threed_ray[1][0],2) + \
					math.pow(center[1] - controller.threed_ray[1][1],2) + \
					math.pow(center[2] - controller.threed_ray[1][2],2) < bestObjectDistance):
					bestObjectDistance = math.pow(center[0] - controller.threed_ray[1][0],2) + \
										  math.pow(center[1] - controller.threed_ray[1][1],2) + \
										  math.pow(center[2] - controller.threed_ray[1][2],2)
					bestObject = i
		bestChain = None
		bestChainDistance = 50000		
		if(bestObject != None):
			for j in Shared.Resi_Dictionary[str(bestObject)].keys():
				extents = Shared.Extent_Dictionary["/" + str(bestObject) + "//" + str(j)]
	
				center = [(extents[0][0] + extents[1][0])/2,
							 (extents[0][1] + extents[1][1])/2,
							 (extents[0][2] + extents[1][2])/2]
				if(math.pow(center[0] - controller.threed_ray[1][0],2) + \
					math.pow(center[1] - controller.threed_ray[1][1],2) + \
					math.pow(center[2] - controller.threed_ray[1][2],2) < bestChainDistance):
					bestChainDistance = math.pow(center[0] - controller.threed_ray[1][0],2) + \
						  math.pow(center[1] - controller.threed_ray[1][1],2) + \
						  math.pow(center[2] - controller.threed_ray[1][2],2)
					bestChain = j

		bestResi = None
		bestResiDistance = 50000
		if(bestChain != None):
			for k in Shared.Resi_Dictionary[str(bestObject)][str(bestChain)]:
				extents = Shared.Extent_Dictionary["/" + str(bestObject) + "//" + str(bestChain) + "/" + str(k)]
				center = [(extents[0][0] + extents[1][0])/2,
							 (extents[0][1] + extents[1][1])/2,
							 (extents[0][2] + extents[1][2])/2]
							 
				if(math.pow(center[0] - controller.threed_ray[1][0],2) + \
					math.pow(center[1] - controller.threed_ray[1][1],2) + \
					math.pow(center[2] - controller.threed_ray[1][2],2) < bestResiDistance):
					bestResiDistance = math.pow(center[0] - controller.threed_ray[1][0],2) + \
							  math.pow(center[1] - controller.threed_ray[1][1],2) + \
								math.pow(center[2] - controller.threed_ray[1][2],2)
					bestResi = k					
		best_object = ("/" + str(bestObject) + "//" + str(bestChain) + "/" + str(bestResi))
	else:
		best_object = controller.threed_cursor_resi
	extents = Shared.Extent_Dictionary[best_object]
	controller.threed_cursor_resi_coords = [(extents[0][0] + extents[1][0])/2,
						 (extents[0][1] + extents[1][1])/2,
						 (extents[0][2] + extents[1][2])/2]
	controller.threed_cursor_resi = best_object
	controller.threed_cursor_disturbed = False



def Translate(controller, axis, distance0, distance1, dest):
	#Each controller must perform its own math for how to manipulate the distances and the axis.
	if not CheckMaster(controller):
		return
	currtime = time.clock()
	if( controller.lastTrans != axis or (controller.lastDist0 < 0 and distance0 > 0) or
										(controller.lastDist0 > 0 and distance0 < 0) or
										(controller.lastDist1 < 0 and distance1 > 0) or
										(controller.lastDist1 > 0 and distance1 < 0)):
		controller.startmovetime = currtime
	delta = currtime - controller.lastmovetime
	controller.lastTrans = axis
	controller.lastDist0 = distance0
	controller.lastDist1 = distance1
	#Update 20x a second to avoid choppy chops. Note that any pymol induced lag will kill this.
	#if (delta < .05):
	#	return

	#Because we're updating distances, we want a maximum speed as well...
	#The first frame after a period of non-movement shouldn't move.
	if (delta > .25):
		delta = 0

	coefficient = currtime - controller.startmovetime

	#Get the current view to manipulate the camera
	#The controller itself should handle the math to determine the distance it changes.
	#This merely translates
	view = list(cmd.get_view())
	if (axis == "Z"):
		distance0 = distance0  * Z_SPEED #*coefficient
		view[11] += distance0 * dest[2]
		
		view[9] += distance0 * dest[0]
		view[10] += distance0 * dest[1]
		
		view[15] -= distance0 * dest[2]
		view[16] -= distance0 * dest[2]
	if (axis == "XY"):
		distance0 = distance0 * XY_SPEED * .3 # * coefficient
		distance1 = distance1 * XY_SPEED * .3 # * coefficient
		#Because X and Y are in relation to camera coordinates, we need to translate the object coordinates properly.
		#The real vector needs to be rotated:
		#Need to rotate by inverse of the camera's rotation matrix to line up X, Y properly with respect to camera
		#Inverse of orthogonal matrix = the transpose of the matrix
		# 0 3 6              0 1 2
		# 1 4 7 inverted is: 3 4 5
		# 2 5 8              6 7 8
		#            X part         Y part               Z part
		x = view[0] * distance0 + view[1] * distance1 #+ view[2] * 0
		y = view[3] * distance0 + view[4] * distance1 #+ view[5] * 0
		z = view[6] * distance0 + view[7] * distance1 #+ view[8] * 0

		view[12] -= x
		view[13] -= y
		view[14] -= z

	cmd.set_view(view)
	controller.lastmovetime = time.clock()
	DamageGUI()


#Angle MUST be in Radians, as any good scientist will use.
def Rotate(controller, angleX, angleY):
	if not CheckMaster(controller):
		return
	currtime = time.clock()
	#Slowdown code vvv - don't use for right now
	#if ((currtime - controller.lastmovetime) < .05):
	#	return
	view = list(cmd.get_view())
	view = Utilities.rotateView(view, angleX, angleY, 0)
	cmd.set_view(view)
	DamageGUI()
	controller.lastmovetime = time.clock()


def Zoom(controller):
	if not CheckMaster(controller):
		return
	cmd.zoom()
	slabWidth=2*(cmd.get_view()[16] - cmd.get_view()[15])
	cmd.clip('slab', slabWidth)

	
#note: this function assumes that all of the coordinates of extent1 are less than those of extent2
def intersectsExtents( extent1, extent2, rayCoord1, rayCoord2):
	# right here, we need to take the ray to the appropriate place in z front and z back
	xfromz1 = rayCoord1[0] + (rayCoord2[0] - rayCoord1[0]) * (extent1[2] - rayCoord1[2]) / (rayCoord2[2] - rayCoord1[2])
	xfromz2 = rayCoord1[0] + (rayCoord2[0] - rayCoord1[0]) * (extent2[2] - rayCoord1[2]) / (rayCoord2[2] - rayCoord1[2])
	yfromz1 = rayCoord1[1] + (rayCoord2[1] - rayCoord1[1]) * (extent1[2] - rayCoord1[2]) / (rayCoord2[2] - rayCoord1[2])
	yfromz2 = rayCoord1[1] + (rayCoord2[1] - rayCoord1[1]) * (extent2[2] - rayCoord1[2]) / (rayCoord2[2] - rayCoord1[2])
	
	xfromy1 = rayCoord1[0] + (rayCoord2[0] - rayCoord1[0]) * (extent1[1] - rayCoord1[1]) / (rayCoord2[1] - rayCoord1[1])
	xfromy2 = rayCoord1[0] + (rayCoord2[0] - rayCoord1[0]) * (extent2[1] - rayCoord1[1]) / (rayCoord2[1] - rayCoord1[1])
	zfromy1 = rayCoord1[2] + (rayCoord2[2] - rayCoord1[2]) * (extent1[1] - rayCoord1[1]) / (rayCoord2[1] - rayCoord1[1])
	zfromy2 = rayCoord1[2] + (rayCoord2[2] - rayCoord1[2]) * (extent2[1] - rayCoord1[1]) / (rayCoord2[1] - rayCoord1[1])

	yfromx1 = rayCoord1[1] + (rayCoord2[1] - rayCoord1[1]) * (extent1[0] - rayCoord1[0]) / (rayCoord2[0] - rayCoord1[0])
	yfromx2 = rayCoord1[1] + (rayCoord2[1] - rayCoord1[1]) * (extent2[0] - rayCoord1[0]) / (rayCoord2[0] - rayCoord1[0])
	zfromx1 = rayCoord1[2] + (rayCoord2[2] - rayCoord1[2]) * (extent1[1] - rayCoord1[1]) / (rayCoord2[1] - rayCoord1[1])
	zfromx2 = rayCoord1[2] + (rayCoord2[2] - rayCoord1[2]) * (extent2[1] - rayCoord1[1]) / (rayCoord2[1] - rayCoord1[1])
	
	# This is not quite right, need to fix...
	return ((((xfromz1 >= extent1[0] and xfromz1 <= extent2[0]) and (yfromz1 >= extent1[1] and yfromz1 <= extent2[1])) or \
		     ((xfromz2 >= extent1[0] and xfromz2 <= extent2[0]) and (yfromz2 >= extent1[1] and yfromz2 <= extent2[1])))    or\
		    (((xfromy1 >= extent1[0] and xfromy1 <= extent2[0]) and (zfromy1 >= extent1[2] and zfromy1 <= extent2[2])) or \
		     ((xfromy2 >= extent1[0] and xfromy2 <= extent2[0]) and (zfromy2 >= extent1[2] and zfromy2 <= extent2[2])))    or\
		    (((yfromx1 >= extent1[1] and yfromx1 <= extent2[1]) and (zfromx1 >= extent1[2] and zfromx1 <= extent2[2])) or \
		     ((yfromx2 >= extent1[1] and yfromx2 <= extent2[1]) and (zfromx2 >= extent1[2] and zfromx2 <= extent2[2]))))
#	return true




def Select(controller, selectUnproject):
	if not CheckMaster(controller):
		return
#	if not Shared.bms.bookmark_mode == Shared.bms.mode_selections:
#		return
	if Shared.bms.showBookmarkDisp:
		return

	#TODO - make this select based on Shared.select_mode
	
	#Selection requires a few bits. Primarily, the
	#Shared.ray[controller.deviceindex]


	AllObjects = cmd.get_names("public_objects")	
	
	recompileExtents()
	
	bestObject = None
	bestObjectDistance = 50000
				
	bestAtom = None
	bestObject = None
	bestChain = None
	bestResi = None
	bestAtomDistance = 50000
	for i in Shared.Resi_Dictionary.keys():
		for j in Shared.Resi_Dictionary[i].keys():
			for l in Shared.Resi_Dictionary[i][j].keys():
				for k in Shared.Resi_Dictionary[i][j][l]:
					extents = Shared.Extent_Dictionary["/" + str(i) + "//" + str(j) + "/" + str(l) + "/" + str(k)]
					center = [(extents[0][0] + extents[1][0])/2,
						  (extents[0][1] + extents[1][1])/2,
						  (extents[0][2] + extents[1][2])/2]
						
					if(math.pow(center[0] - selectUnproject[0],2) + \
						   math.pow(center[1] - selectUnproject[1],2) + \
						   math.pow(center[2] - selectUnproject[2],2) < bestAtomDistance):
						bestAtomDistance = math.pow(center[0] - selectUnproject[0],2) + \
						    math.pow(center[1] - selectUnproject[1],2) + \
						    math.pow(center[2] - selectUnproject[2],2)
						bestAtom = k
						bestChain = j
						bestResi = l
						bestObject = i
	
	if(Shared.select_mode == 0):
		best_object = ("/" + str(bestObject) + "//" + str(bestChain) + "/" + str(bestResi) + "/" + str(bestAtom))
	elif Shared.select_mode == 1 :
		best_object = ("/" + str(bestObject) + "//" + str(bestChain) + "/" + str(bestResi))
	elif Shared.select_mode == 2 :
		best_object = ("/" + str(bestObject) + "//" + str(bestChain))
	elif Shared.select_mode == 3 :
		best_object = ("/" + str(bestObject))
	

	print best_object
	Shared.lastSelected = best_object
	if best_object != None and "WM_Sel" in cmd.get_names("all"):
		#cmd.delete("WM_Sel")
		cmd.select("WM_Sel","( WM_Sel or " + str(best_object)+ ") and not ( WM_Sel and " + str(best_object)+ ")")
	elif best_object != None :
		cmd.select("WM_Sel",best_object)

def MarkMasterUsed(controller):
	Shared.master_acted = True
	Shared.master_timedout = False


def UpdateGUI():
	#Force a GUI update to reflect the change in controls...
	if (Shared.gui_damaged == True):
		Shared.gui_damaged = False
	cmd.refresh()


def DamageGUI():
	Shared.gui_damaged = True



#Switch between selection levels - Atom, Residue, Chain, Object
def SelectModeUp(controller):
	if not CheckMaster(controller):
		return
	if (Shared.bms.showBookmarkDisp):
		return		
	if (Shared.select_mode == Shared.SELECT_ATOM):
		return
	Shared.select_mode -= 1
	print "select mode is now:",
	if(Shared.select_mode == 0):
		print "atoms"
	if(Shared.select_mode == 1):
		print "residues"
	if(Shared.select_mode == 2):
		print "chains"		
	if(Shared.select_mode == 3):
		print "objects"
	DamageGUI()

def SelectModeDown(controller):
	if not CheckMaster(controller):
		return
	if (Shared.bms.showBookmarkDisp):
		return		
	if (Shared.select_mode == Shared.SELECT_OBJECT):
		return
	Shared.select_mode += 1
	print "select mode is now:",
	if(Shared.select_mode == 0):
		print "atoms"
	if(Shared.select_mode == 1):
		print "residues"
	if(Shared.select_mode == 2):
		print "chains"		
	if(Shared.select_mode == 3):
		print "objects"
	DamageGUI()
	
	
def ToggleBookmarkMode(controller):
	if not CheckMaster(controller):
		return
	if (Shared.bms.showBookmarkDisp == True):
		return
	if (Shared.bms.bookmark_mode == Shared.bms.mode_selections):
		Shared.bms.bookmark_mode = Shared.bms.mode_views
	else:
		Shared.bms.bookmark_mode = Shared.bms.mode_selections
	DamageGUI()

def SelectionModeLeft(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_selections:
		return

	num_rec_sel = Shared.bms.num_rec_sel
	if (num_rec_sel > 0):
		num_rec_sel -= 1

	Shared.bms.num_rec_sel = num_rec_sel

 	if num_rec_sel >= 0:
		cmd.delete("WM_Sel")
		[chain,resn,resi] = Shared.bms.recorded_sel[num_rec_sel]
		cmd.select("WM_Sel","chain " + chain + " and resi " + resi + " and resn " + resn)
		Shared.bms.moved_from_sel = False


def SelectionModeRight(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_selections:
		return

	num_rec_sel = Shared.bms.num_rec_sel
	if (num_rec_sel < len(Shared.bms.recorded_sel)-1):
		num_rec_sel += 1
	Shared.bms.num_rec_sel = num_rec_sel
	if num_rec_sel >= 0:
		cmd.delete("WM_Sel")
		[chain,resn,resi] = Shared.bms.recorded_sel[num_rec_sel]
		cmd.select("WM_Sel","chain " + chain + " and resi " + resi + " and resn " + resn)
		Shared.bms.moved_from_sel = False

#Master Controller Functions:

def ToggleMaster(controller):
	
	if controller.master == True:
		ReleaseMaster(controller)
	else:
		controller.master = True


def ReleaseMaster(controller):
	for other_controller in Shared.controller_list:
		if other_controller.master == False:
			other_controller.AlertMasterFree()   #Buzz/whatever everyone else.
	controller.master = False
	DamageGUI()
	pass


#Pointer Enable/Disable Functions:
#TODO - maybe delete this.
def TogglePointer(controller):
	if controller.pointing == True:
		DisablePointer(controller)
	else:
		EnablePointer(controller)
	DamageGUI()

def EnablePointer(controller):
	controller.pointing = True
	#print "pointer on!"

def DisablePointer(controller):
	controller.pointing = False

#BookMark Functions

def CreateSelection(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_selections:
		return

	names = cmd.get_names("selections")
	selection = ""
	for i in range(0,len(names)):
		selection += names[i] + ","

	atoms = None

	if selection != "":
		atoms = cmd.get_model(selection)
	else:
		return

	if len(atoms.atom) > 0:
		# Check that the selection is not bookmarked aleready
		already_bookmarked = False
		for i in range(0,len(Shared.bms.recorded_sel)):
			if Shared.bms.recorded_sel[i][0] == atoms.atom[0].chain \
			and Shared.bms.recorded_sel[i][1] == atoms.atom[0].resn \
			and Shared.bms.recorded_sel[i][2] == atoms.atom[0].resi:
				already_bookmarked = True

		if not already_bookmarked:
			Shared.bms.recorded_sel.append([atoms.atom[0].chain,atoms.atom[0].resn,atoms.atom[0].resi])
			[xG,yG,zG] = Utilities.barycenterSelection()
			resn = atoms.atom[0].resn
			resi = atoms.atom[0].resi
			index = atoms.atom[0].index
			SelectionMark.addMark(resn,resi,index,xG,yG,zG)
			#cmd.enable("WM_Sel")
			DamageGUI()
			master_acted = True

			# If the GUI is currently between the previous last and the new last bookmark
			# and we haven't selected something else yet, we have to show that we are on the new last bookmark
			if Shared.bms.moved_from_sel and Shared.bms.num_rec_sel + 1 == len(Shared.bms.recorded_sel) - 1:
				Shared.bms.moved_from_sel = False
				Shared.bms.num_rec_sel += 1

def DeleteSelection(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.moved_from_sel and len(Shared.bms.recorded_sel)>0:
		cmd.delete("WM_Sel")
		resn = Shared.bms.recorded_sel[Shared.bms.num_rec_sel][1]
		resi = Shared.bms.recorded_sel[Shared.bms.num_rec_sel][2]
		SelectionMark.delMark(resn, resi)

		Shared.bms.recorded_sel.pop(Shared.bms.num_rec_sel)
		if Shared.bms.num_rec_sel > 0:
			Shared.bms.num_rec_sel -= 1


def CreateBookmarkView(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_views:
		return
	if not Shared.bms.showBookmarkDisp:
		view = list(cmd.get_view())
		Shared.bms.getViewForBookmark = True
		Shared.bms.recorded_views.append(view)
		Shared.bms.total_rec_views = len(Shared.bms.recorded_views)
	
		# If the GUI is currently between the previous last and the new last bookmark
		# and we haven't moved yet, we have to show that we are on the new last bookmark
		#	if Shared.bms.moved_from_view and (Shared.bms.total_rec_views == 1 or Shared.bms.num_rec_view + 1 == Shared.bms.total_rec_views - 1):
		#		Shared.bms.moved_from_view = False
		if (Shared.bms.total_rec_views == 1 or Shared.bms.num_rec_view + 1 == Shared.bms.total_rec_views - 1):
			if Shared.bms.num_rec_view + 1 == Shared.bms.total_rec_views - 1:
				Shared.bms.num_rec_view += 1

def DeleteBookmarkView(controller):
	#TODO: this.
	if not CheckMaster(controller):
		return
	if len(Shared.bms.recorded_views)>0:
		Shared.bms.recorded_views.pop(Shared.bms.num_rec_view)
		Shared.bms.recorded_view_pics.pop(Shared.bms.num_rec_view)
		Shared.bms.recorded_view_width.pop(Shared.bms.num_rec_view)
		Shared.bms.recorded_view_height.pop(Shared.bms.num_rec_view)
		if Shared.bms.num_rec_view > 0:
			Shared.bms.num_rec_view -= 1
		Shared.bms.total_rec_views -=1
	pass
	
def ToggleBookmarkView(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_views:
		return
	Shared.bms.showBookmarkDisp = not Shared.bms.showBookmarkDisp
	if(Shared.bms.showBookmarkDisp):
		Shared.hide_pointers = True
		Shared.bookmarkScreenChanged = True
	else:
		Shared.hide_pointers = False
	print Shared.bms.showBookmarkDisp


#Original Selection code moved here


def CenterOnSelection(controller):
	if not CheckMaster(controller):
		return
	# Bring selection on the center/front
    # Find barycenter of the selection
	[xG,yG,zG] = Utilities.barycenterSelection()

	# Transform the coordinates to the camera frame (just rotation, no translation)
	view = list(cmd.get_view())

	#current_rot = matrix([[view[0],view[1],view[2]],[view[3],view[4],view[5]],[view[6],view[7],view[8]]])
	#inv_current_rot = linalg.inv(current_rot)
	#coord = inv_current_rot*matrix([[xG],[yG],[zG]])
	# Compute the transformation to go from the current view
	# to a view where the vector OG is aligned with z
	#r = pow(pow(coord[0,0],2) + pow(coord[1,0],2) + pow(coord[2,0],2),0.5)
	r = 1

	theta = math.acos(coord[1,0]/r)
	cotheta = math.pi/2 - theta

	phi = math.atan2(coord[0,0],coord[2,0])

	# Rotation y,phi then x,coteta
	poi = Utilities.rotateView(view,-cotheta,phi,0)
	poi[9] = 0.0

	#		Shared.bms.moved_from_view = True

	Shared.mode = "Moving selection"
	transitionToPOI(view,poi)
	Shared.mode = "-"
	DamageGUI()
	master_acted = True


def toggleCameras():
	if(Shared.camera.visible):
		cmd.disable('Cameras')
	else:
		cmd.enable('Cameras')

def movViewLeft(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_views:
		return
	if not Shared.bms.showBookmarkDisp:
		return

	this_index = Shared.bms.num_rec_view
	screen_number = Shared.bms.num_rec_view/(Shared.bms.max_height*Shared.bms.max_width)
	starting_index = screen_number*Shared.bms.max_height*Shared.bms.max_width
	finish_index = Shared.bms.total_rec_views - 1
	if( finish_index - starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
		finish_index = starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
	number_rows = int(math.ceil(float(finish_index - starting_index + 1) / float(Shared.bms.max_width)))
	row_quantity = []
	num_remain = finish_index - starting_index + 1
	rows_remain = number_rows
	this_row = -1
	for x in range(number_rows):
		row_quantity.append(math.ceil( float(num_remain) / float(rows_remain)))
		num_remain -= row_quantity[x]
		if(num_remain <= finish_index - this_index and this_row ==-1):
			this_row = x
		rows_remain -= 1
	indCount = 0
	for a in range(int(this_row)):
		indCount += row_quantity[a]
	#if there's somewhere to go on this row, just move left in the row.
	if(indCount + starting_index < this_index):
		Shared.bms.num_rec_view-=1
	#otherwise, we'll have to figure out how many to move back in the previous screen.
	else:
		prev_screen = screen_number - 1
		if(prev_screen < 0):
			prev_screen = math.ceil(float(Shared.bms.total_rec_views) / (Shared.bms.max_height*Shared.bms.max_width)) - 1
		#so we want to then get to the last element in the previous screen's row that is the same as this row on our screen.
		#so first we have to compute everything we need about that screen's stuff:
		prev_starting_index = prev_screen*Shared.bms.max_height*Shared.bms.max_width
		prev_finish_index = Shared.bms.total_rec_views - 1
		if( prev_finish_index - prev_starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
			prev_finish_index = prev_starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
		prev_number_rows = math.ceil(float(prev_finish_index - prev_starting_index + 1) / float(Shared.bms.max_width))
		num_remain = prev_finish_index - prev_starting_index + 1
		rows_remain = prev_number_rows
		prev_indCount = 0
		for x in range(int(min([this_row + 1, prev_number_rows]))):
			prev_indCount += (math.ceil(num_remain / rows_remain ))
			num_remain -= (math.ceil(num_remain / rows_remain ))
			rows_remain -= 1
		#now the prev_indcount holds the number of indices in the first row(s) of the previous screen. we simply need to shift the selection to prev_starting_index + prev_indcount - 1
		Shared.bms.num_rec_view = int(prev_starting_index + prev_indCount - 1)
		Shared.bookmarkScreenChanged = True
		#whew! done. that's what you get for dynamic rows and screens i guess.
					
			
def movViewRight(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_views:
		return
	if not Shared.bms.showBookmarkDisp:
		return
	#much the same as moving left, only we move to the /next screen.
	this_index = Shared.bms.num_rec_view
	screen_number = Shared.bms.num_rec_view/(Shared.bms.max_height*Shared.bms.max_width)
	starting_index = screen_number*Shared.bms.max_height*Shared.bms.max_width
	finish_index = Shared.bms.total_rec_views - 1
	if( finish_index - starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
		finish_index = starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
	number_rows = math.ceil(float(finish_index - starting_index + 1) / float(Shared.bms.max_width))
	row_quantity = []
	num_remain = finish_index - starting_index + 1
	rows_remain = number_rows
	this_row = -1
	for x in range(int(number_rows)):
		row_quantity.append(math.ceil(num_remain / rows_remain ))
		num_remain -= row_quantity[x]
		if(num_remain <= finish_index - this_index and this_row == -1):
			this_row = x
		rows_remain -= 1
	indCount = 0
	for a in range(int(this_row + 1)):
		indCount += row_quantity[a]
	#if there's somewhere to go on this row, just move left in the row.
	if(indCount + starting_index - 1 > this_index):
		Shared.bms.num_rec_view+=1
	#otherwise, we'll have to figure out how many to move back in the next screen.
	else:
		next_screen = screen_number + 1
		if(next_screen > math.ceil(float(Shared.bms.total_rec_views) / (Shared.bms.max_height*Shared.bms.max_width)) - 1):
			next_screen = 0
		#so we want to then get to the last element in the previous screen's row that is the same as this row on our screen.
		#so first we have to compute everything we need about that screen's stuff:
		next_starting_index = next_screen*Shared.bms.max_height*Shared.bms.max_width
		next_finish_index = Shared.bms.total_rec_views - 1
		if( next_finish_index - next_starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
			next_finish_index = next_starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
		next_number_rows = math.ceil(float(next_finish_index - next_starting_index + 1) / float(Shared.bms.max_width))
		num_remain = next_finish_index - next_starting_index + 1
		rows_remain = next_number_rows
		next_indCount = 0
		for x in range(int(min([this_row,next_number_rows - 1]))):
			next_indCount += (math.ceil(num_remain / rows_remain ))
			num_remain -= (math.ceil(num_remain / rows_remain ))
			rows_remain -= 1
		#now the prev_indcount holds the number of indices in the first row(s) of the next screen. we simply need to shift the selection to next_starting_index + next_indcount
		Shared.bms.num_rec_view = int(next_starting_index + next_indCount)
		Shared.bookmarkScreenChanged = True

def movViewDown(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_views:
		return
	if not Shared.bms.showBookmarkDisp:
		return
	this_index = Shared.bms.num_rec_view
	screen_number = Shared.bms.num_rec_view/(Shared.bms.max_height*Shared.bms.max_width)
	starting_index = screen_number*Shared.bms.max_height*Shared.bms.max_width
	finish_index = Shared.bms.total_rec_views - 1
	if( finish_index - starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
		finish_index = starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
	number_rows = math.ceil(float(finish_index - starting_index + 1) / float(Shared.bms.max_width))
	row_quantity = []
	num_remain = finish_index - starting_index + 1
	rows_remain = number_rows
	this_row_index = -1
	this_row = -1
	for x in range(int(number_rows)):
		row_quantity.append(math.ceil(num_remain / rows_remain ))
		num_remain -= row_quantity[x]
		rows_remain -= 1
		if(num_remain <= finish_index - this_index and this_row == -1):
			this_row = x
			thus_far = finish_index + 1 - num_remain - row_quantity[x]
			#this starts counting at 0
			this_row_index = this_index - thus_far

	#alright, now we have to move to roughly the same position that we are in on the next row
	# this means we want to go to (in row indices) this_row_index * (this_row_count/next_row_count) in the next row
	# so the index will be incremented by the number required to get to the next row (this_row_count - this_row_index + 1) added to the next row index we want (above)
	if(this_row < number_rows - 1):
		Shared.bms.num_rec_view += int((row_quantity[this_row] - this_row_index) + math.ceil((this_row_index + 1) * (row_quantity[this_row + 1] / row_quantity[this_row])) - 1)
	#luckily if we need to change pages, we only need to know about the first row of the next page.
	# and then we use the same calculation as above.
	else:
		next_screen = screen_number + 1
		if(next_screen > math.ceil(float(Shared.bms.total_rec_views) / (Shared.bms.max_height*Shared.bms.max_width)) - 1):
			next_screen = 0
		starting_index = next_screen*Shared.bms.max_height*Shared.bms.max_width
		finish_index = Shared.bms.total_rec_views - 1
		if( finish_index - starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
			finish_index = starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
		number_rows = math.ceil(float(finish_index - starting_index + 1) / float(Shared.bms.max_width))
		num_remain = finish_index - starting_index + 1
		rows_remain = number_rows
		first_row_quantity = math.ceil(num_remain / rows_remain)
		Shared.bms.num_rec_view = int(starting_index + math.ceil((this_row_index + 1) * (first_row_quantity / row_quantity[this_row])) - 1)
		Shared.bookmarkScreenChanged = True

def movViewUp(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_views:
		return
	if not Shared.bms.showBookmarkDisp:
		return
	this_index = Shared.bms.num_rec_view
	screen_number = Shared.bms.num_rec_view/(Shared.bms.max_height*Shared.bms.max_width)
	starting_index = screen_number*Shared.bms.max_height*Shared.bms.max_width
	finish_index = Shared.bms.total_rec_views - 1
	if( finish_index - starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
		finish_index = starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
	number_rows = math.ceil(float(finish_index - starting_index + 1) / float(Shared.bms.max_width))
	row_quantity = []
	num_remain = finish_index - starting_index + 1
	rows_remain = number_rows
	this_row_index = -1
	this_row = -1
	for x in range(int(number_rows)):
		row_quantity.append(math.ceil(num_remain / rows_remain ))
		num_remain -= row_quantity[x]
		rows_remain -= 1
		if(num_remain <= finish_index - this_index and this_row == -1):
			this_row = x
			thus_far = finish_index + 1 - num_remain - row_quantity[x]
			#this starts counting at 0
			this_row_index = this_index - thus_far
	#alright, now we have to move to roughly the same position that we are in on the next row
	# this means we want to go to (in row indices) this_row_index * (this_row_count/prev_row_count) in the next row
	# so the index will be incremented by the number required to get to the previous row -( this_row_index + 1) added to the previous row index we want (above)
	if(this_row > 0):
		Shared.bms.num_rec_view -= int((this_row_index + 1) + row_quantity[this_row - 1] - math.floor((this_row_index + 1) * (row_quantity[this_row - 1] / row_quantity[this_row])))
	#luckily if we need to change pages, we only need to know about the last row of the next page.
	# and then we use the same calculation as above.
	else:
		prev_screen = screen_number - 1
		if(prev_screen < 0 ):
			prev_screen = math.ceil(float(Shared.bms.total_rec_views) / (Shared.bms.max_height*Shared.bms.max_width)) - 1
		starting_index = prev_screen*Shared.bms.max_height*Shared.bms.max_width
		finish_index = Shared.bms.total_rec_views - 1
		if( finish_index - starting_index > Shared.bms.max_height*Shared.bms.max_width - 1):
			finish_index = starting_index + Shared.bms.max_height*Shared.bms.max_width - 1
		number_rows = int(math.ceil(float(finish_index - starting_index + 1) / float(Shared.bms.max_width)))
		num_remain = finish_index - starting_index + 1
		rows_remain = number_rows
		row_quantity_prev = []
		for x in range(number_rows):
			row_quantity_prev.append(math.ceil(num_remain / rows_remain ))
			num_remain -= row_quantity_prev[x]
			rows_remain -= 1
		Shared.bms.num_rec_view = int(finish_index - row_quantity_prev[number_rows - 1] + math.ceil((this_row_index + 1) * (row_quantity_prev[number_rows - 1]/ row_quantity[this_row] )))
		Shared.bookmarkScreenChanged = True


def MoveToBookmark(controller):
	if not CheckMaster(controller):
		return
	if not Shared.bms.bookmark_mode == Shared.bms.mode_views:
		return
	if not Shared.bms.showBookmarkDisp:
		return
	Shared.bms.showBookmarkDisp = False
	Shared.hide_pointers = False
	eye = list(cmd.get_view())
	poi = Shared.bms.recorded_views[Shared.bms.num_rec_view]
	transitionToPOI(eye,poi)
	DamageGUI()


# Transition to "Point of Interest"
def transitionToPOI(eye,poi,k=0.5):
	Shared.busy = True
	Utilities.transitionToPOI(eye,poi,k)
	Shared.busy = False
	
def startAccelerometerNavigation(controller):
	if (not CheckMaster(controller)):
		return
	controller.acc_nav_state = [0,0,0]
	controller.prev_accelerometer_navigation[0] = (controller.button_state.accelX * 4 + controller.button_state.extraX1 * 2 + controller.button_state.extraX2)/(50)
	controller.prev_accelerometer_navigation[1] = (controller.button_state.accelY * 4 + controller.button_state.extraY * 2)/(50)
	controller.prev_accelerometer_navigation[2] = 0#(controller.button_state.accelZ * 4 + controller.button_state.extraZ * 2)/(50)
	controller.acc_nav_state = controller.prev_accelerometer_navigation
	controller.accelerometer_view = list(cmd.get_view())
		
def accelerometerNavigation(controller,weight):
	if (not CheckMaster(controller)):
		return
	view = list(cmd.get_view())
	relX = ((((controller.button_state.accelX * 4 + controller.button_state.extraX1 * 2 + controller.button_state.extraX2)/(50)) * weight + controller.prev_accelerometer_navigation[0])/(1 + weight))
	relY = ((((controller.button_state.accelY * 4 + controller.button_state.extraY * 2)/(50)) * weight + controller.prev_accelerometer_navigation[1])/(1 + weight))
	relZ = 0#((((controller.button_state.accelZ * 4 + controller.button_state.extraZ * 2)/(50)) * weight + controller.prev_accelerometer_navigation[2])/(1 + weight))
	changeX = (relX - controller.acc_nav_state[0])/2
	changeY = (relY - controller.acc_nav_state[1])/2
	changeZ = (relZ - controller.acc_nav_state[2])/2
	view = controller.accelerometer_view

	view = Utilities.rotateView(view, changeY, changeZ, changeX)
	controller.prev_accelerometer_navigation = [relX, relY, relZ]
	cmd.set_view(view)
	DamageGUI()
	
