from pymol import cmd
from pymol.opengl.gl import *
from pymol.opengl.glu import *
from pymol.opengl.glut import *

import math
import time
		
def multMatrices(m1,m2):
	if len(m1) == 9 and len(m2) == 9:
		res = [0,0,0,0,0,0,0,0,0]
		for i in range(0,3):
			for j in range(0,3):
				res[3*i+j] = m1[3*i]*m2[j] + m1[3*i+1]*m2[j+3] + m1[3*i+2]*m2[j+6]
				
		return res

		
def drawLabel(text):
	for c in text:
		glutStrokeCharacter(GLUT_STROKE_ROMAN,ord(c))
		
	
def matToQuat(m):
	q = [0,0,0,0]
	q[0] = math.sqrt( max( 0, 1 + m[0] + m[4] + m[8] ) ) / 2
	q[1] = math.sqrt( max( 0, 1 + m[0] - m[4] - m[8] ) ) / 2
	q[2] = math.sqrt( max( 0, 1 - m[0] + m[4] - m[8] ) ) / 2
	q[3] = math.sqrt( max( 0, 1 - m[0] - m[4] + m[8] ) ) / 2
	
	if (math.fabs(m[7] - m[5]) > 0):
		q[1] = q[1] * (m[7] - m[5]) / math.fabs(m[7] - m[5])
	if (math.fabs(m[2] - m[6]) > 0):
		q[2] = q[2] * (m[2] - m[6]) / math.fabs(m[2] - m[6])
	if (math.fabs(m[3] - m[1]) > 0):
		q[3] = q[3] * (m[3] - m[1]) / math.fabs(m[3] - m[1])
	
	return q
	

def quatToMat(q):
	m = [1,0,0,0,1,0,0,0,1]

	W = q[0]
	X = q[1]
	Y = q[2]
	Z = q[3]

	xx = X * X
	xy = X * Y
	xz = X * Z
	xw = X * W

	yy = Y * Y
	yz = Y * Z
	yw = Y * W

	zz = Z * Z
	zw = Z * W

	m[0] = 1 - 2 * ( yy + zz )
	m[1] = 2 * ( xy - zw )
	m[2] = 2 * ( xz + yw )

	m[3] = 2 * ( xy + zw )
	m[4] = 1 - 2 * ( xx + zz )
	m[5] = 2 * ( yz - xw )

	m[6] = 2 * ( xz - yw )
	m[7] = 2 * ( yz + xw )
	m[8] = 1 - 2 * ( xx + yy )

	return m


def slerp(qA,qB,t):
	# quaternion to return
	qM = [0,0,0,0]
	
	# Calculate angle between them.
	cosHalfTheta = qA[0] * qB[0] + qA[1] * qB[1] + qA[2] * qB[2] + qA[3] * qB[3]
	
	# if qA=qB or qA=-qB then theta = 0 and we can return qA
	if abs(cosHalfTheta) >= 1.0:
		return qA
	
	# Calculate temporary values.
	halfTheta = math.acos(cosHalfTheta);
	sinHalfTheta = math.sqrt(1.0 - cosHalfTheta*cosHalfTheta);
	
	# if theta = 180 degrees then result is not fully defined
	# we could rotate around any axis normal to qA or qB
	if math.fabs(sinHalfTheta) < 0.001:
		for i in range(0,4):
			qM[i] = (qA[i] * 0.5 + qB[i] * 0.5)
			
		return qM
		
	ratioA = math.sin((1 - t) * halfTheta) / sinHalfTheta
	ratioB = math.sin(t * halfTheta) / sinHalfTheta
	
	# calculate Quaternion.
	for i in range(0,4):
		qM[i] = (qA[i] * ratioA + qB[i] * ratioB)
	
	return qM
	

# Transition to "Point of Interest"
def transitionToPOI(eye,poi,k):	
	t = 0.03
	eyeRot = [1,0,0,0,1,0,0,0,1]
	for i in range(0,9):
			eyeRot[i] = eye[i]
	eyeOrig = [eye[9],eye[10],eye[11],eye[12],eye[13],eye[14],eye[15],eye[16]]
	poiRot = [1,0,0,0,1,0,0,0,1]
	for i in range(0,9):
			poiRot[i] = poi[i]
			
	rotQuat = matToQuat(eyeRot)
	poiQuat = matToQuat(poiRot)
	transition_over = False
	
		
	while not transition_over:
		
			
		
		t = t - k*(t-1)*t
		
		eye[9] = eyeOrig[0] - t*(eyeOrig[0] - poi[9])
		eye[10] = eyeOrig[1] - t*(eyeOrig[1] - poi[10])
		eye[11] = eyeOrig[2] - t*(eyeOrig[2] - poi[11])
		
		eye[12] = eyeOrig[3] - t*(eyeOrig[3] - poi[12])
		eye[13] = eyeOrig[4] - t*(eyeOrig[4] - poi[13])
		eye[14] = eyeOrig[5] - t*(eyeOrig[5] - poi[14])
		
		eye[15] = eyeOrig[6] - t*(eyeOrig[6] - poi[15])
		eye[16] = eyeOrig[7] - t*(eyeOrig[7] - poi[16])
		
		tQuat = slerp(rotQuat,poiQuat,t)
		tRot = quatToMat(tQuat)
		
		
		for i in range(0,9):
			eye[i] = tRot[i]
		
		cmd.set_view(eye)
		
		time.sleep(0.1)
		
		transition_over = True
		if abs(eye[9]-poi[9]) > 0.1				\
			or abs(eye[10]-poi[10]) > 0.1		\
			or abs(eye[11]-poi[11]) > 0.08		\
			or t < 0.99:
			
			transition_over = False
			
			

def rotateView(mat,angleX,angleY,angleZ):
	if len(mat) >= 9:	
		rotX = [1,0,0, 0,math.cos(angleX),-math.sin(angleX), 0,math.sin(angleX),math.cos(angleX)]
		rotY = [math.cos(angleY),0,math.sin(angleY), 0,1,0, -math.sin(angleY),0,math.cos(angleY)]
		rotZ = [math.cos(angleZ),-math.sin(angleZ),0, math.sin(angleZ),math.cos(angleZ),0, 0,0,1]

		rotXYZ = multMatrices(multMatrices(rotZ,rotY),rotX)
		rot_prev = []
		for i in range(0,9):
			rot_prev.append(mat[i])
			
		rot_new = multMatrices(rot_prev,rotXYZ)

		mat_new = []

		for i in range(0,9):
			mat_new.append(rot_new[i])
		for i in range(9,len(mat)):
			mat_new.append(mat[i])
			
		return mat_new
		
		
def barycenterSelection():
	names = cmd.get_names("selections")
	selection = ""
	for i in range(0,len(names)):
		selection += names[i] + ","
		
	atoms = cmd.get_model(selection)
	
	xG = 0
	yG = 0
	zG = 0
	
	for at in atoms.atom:
		xG += at.coord[0]	
		yG += at.coord[1]	
		zG += at.coord[2]
	
	
	if len(atoms.atom) > 0:
		xG /= len(atoms.atom)
		yG /= len(atoms.atom)
		zG /= len(atoms.atom)
	
	return [xG,yG,zG]
