#include "stdafx.h"

#include "WiimoteManager.h"
#include <math.h>
#include <iostream>




WiimoteManager::WiimoteManager()
:m_nbConnected(0)
{
	for(int i=0; i<NB_MAX_WIIMOTES; i++)
	{
		m_wiimotes[i] = new wiimote();
		m_buttonsStates[i] = new WiimoteButtonsStates(m_wiimotes[i]);
		m_kalmanFilters[i] = new KalmanFilter();
	}

	for(int i=0; i<NB_MAX_WIIMOTES; i++)
	{
		if(m_wiimotes[i]->Connect(wiimote::FIRST_AVAILABLE))
		{
			m_nbConnected++;
			
			if(i < 4)
			{
				unsigned char leds = pow(2,(float)i);
				m_wiimotes[i]->SetLEDs(leds);
			}

			m_wiimotes[i]->SetReportType(wiimote::IN_BUTTONS_ACCEL_IR_EXT);
			//std::cout << wiimote::Load16bitMonoSampleWAV("becomeMaster.wav",m_masterSample) << std::endl;

			m_kalmanFilters[i]->predictionBegin(0.0,0.0);

			if(m_wiimotes[i]->NunchukConnected())
			{
				m_wiimotes[i]->Nunchuk.Joystick.DeadZone.X = (float)0.1;
				m_wiimotes[i]->Nunchuk.Joystick.DeadZone.Y = (float)0.1;
			}
		}
	}
}


WiimoteManager::~WiimoteManager()
{
	

}


void WiimoteManager::update()
{
	for (unsigned int i=0; i<m_nbConnected; i++)
	{
		m_wiimotes[i]->RefreshState();
		m_buttonsStates[i]->updateButtonsStates();

		// IR
		unsigned int dot0X	= m_wiimotes[i]->IR.Dot[0].RawX;
		unsigned int dot0Y	= m_wiimotes[i]->IR.Dot[0].RawY;
		unsigned int dot1X	= m_wiimotes[i]->IR.Dot[1].RawX;
		unsigned int dot1Y	= m_wiimotes[i]->IR.Dot[1].RawY;

		float xC = (dot0X + dot1X)/2.0;
		float yC = (dot0Y + dot1Y)/2.0;

		m_kalmanFilters[i]->predictionUpdate(xC,yC);
		m_kalmanFilters[i]->predictionReport();
		CvMat* position = m_kalmanFilters[i]->getState();

		m_cursorsPositions[i][0] = position->data.fl[0];
		m_cursorsPositions[i][1] = position->data.fl[1];

		// Nunchuk
		if(m_wiimotes[i]->NunchukConnected())
		{
			m_buttonsStates[i]->updateNunchukStates();
		}
	}
}
